#========================================================================================
# §package      DE_EPAGES::Catalog::API::Resource::CategoriesTree
# §state        public
# §base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# §description  implements the REST resource for a CategoriesTree
#========================================================================================
package DE_EPAGES::Catalog::API::Resource::CategoriesTree;
use base DE_EPAGES::WebService::API::Resource::BaseObjectResource;

use DE_EPAGES::Core::API::Url qw ( ParseQueryString );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::WebService::API::Util qw (ReadJSONBoolean);
use feature 'switch';
use JSON;

use strict;



sub service {
    return LoadObjectByPath('/RestServices/CategoriesTree');
}



sub class {
    return 'Shop';
}



sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        GET => [
            ['get', 'application/json'],
            ['get', 'application/vnd.epages.v1+json'],
        ],
    }->{$RequestMethod};
}



sub match {
    my $self = shift;
    my ($Request) = @_;

    return 1 if $Request->uri->path =~ m|^/api/categories-tree$|;
    return $1;
}



sub requestContext {
    my $self = shift;
    my ( $Site, $GUID, $Request ) = @_;

    return $self->SUPER::requestContext( $Site, $GUID, $Request );
}



sub params {
    my $self = shift;
    my ( $ActionName, $Object, $Request ) = @_;

    given ($ActionName) {
        when ('get') {
            my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
            my $IncludeInvisible = $hQuery->{'includeInvisible'}->[0];
            my $LocaleID = $hQuery->{'locale'}->[0];
            return [ $Object, $LocaleID, $IncludeInvisible ];
        }
        default {
            return [$Object];
        }
    }
}


sub get {
    my $self = shift;
    my ( $Shop, $LocaleID, $IncludeInvisible ) = @_;

    my $Context = $self->_createShopRequestContext($Shop);
    $LocaleID = $Context->validLocale($LocaleID);

    my $Locale = $self->_loadLocale($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');

    $IncludeInvisible = ReadJSONBoolean($IncludeInvisible);
    my $RootCat = $Shop->get('Categories');
    my $hInfo = GetCatInfoAndRecurs($RootCat, $LanguageID, $IncludeInvisible);

    return $hInfo;
}

#-- Recursively build a category's tree based on it and its subcategories
sub GetCatInfoAndRecurs {
  my ($Cat, $LanguageID, $IncludeInvisible) = @_;

  my $hInfo = {
    'categoryId' => $Cat->get('GUID'),
    'objectId' => $Cat->get('ID'),
    'name' => $Cat->get('NameOrAlias', $LanguageID),
    'visible' => $Cat->get('IsVisible') ? JSON::XS::true : JSON::XS::false,
    'visibleInNavigation' => $Cat->get('VisibleInNavigation') ? JSON::XS::true : JSON::XS::false,
    'mboUrl' => $Cat->get('WebUrlAdminSSL'),
  };

  my $aSubCats = $Cat->get('SubCategories');
  my $aSubCatsInfo = [];

  foreach my $SubCat ( @$aSubCats ) {
    #-- If you have not asked for invisible categories and this one is invisible, skip!
    next if !$IncludeInvisible && !$SubCat->get('IsVisible');

    my $hSubCatInfo = GetCatInfoAndRecurs($SubCat, $LanguageID, $IncludeInvisible);
    push @$aSubCatsInfo, $hSubCatInfo = $hSubCatInfo if scalar keys %$hSubCatInfo > 0;
  }
  $hInfo->{'subCategories'} = $aSubCatsInfo if scalar @$aSubCatsInfo > 0;

  return $hInfo;
}

sub _loadLocale {
    my $self = shift;
    my ($LocaleID) = @_;
    return DE_EPAGES::Object::API::Object::Locale->load($LocaleID);
}

sub _createShopRequestContext {
    my $self = shift;
    my ($Shop) = @_;
    return DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
}


1;
