#==============================================================================
# package      DE_EPAGES::GoogleSmartShopping::UI::ProductFeed
# state        private
#==============================================================================
package DE_EPAGES::GoogleSmartShopping::UI::ProductFeed;
use base DE_EPAGES::ProductPortal::UI::PortalConfig;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use HTTP::Status qw ( HTTP_OK );

use DE_EPAGES::ProductPortal::API::Object::PortalConfig;
use DE_EPAGES::Object::API::Table::Language qw (GetInfoLanguage);
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping );
use DE_EPAGES::GoogleSmartShopping::API::GoogleApiService;

use DE_EPAGES::GoogleSmartShopping::API::ProductBatchService;
use DE_EPAGES::GoogleSmartShopping::API::Object::PagerMerchantCenterProducts;
use DE_EPAGES::Product::API::Object::PagerProducts;
use DE_EPAGES::GoogleSmartShopping::API::GooglePortalService;
use strict;

use constant AMOUNT_OF_PRODUCTS_PER_BATCH => 500;

#========================================================================================
# function     RemoveProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       RemoveProducts( $Servlet );
#----------------------------------------------------------------------------------------
# description  Removes Products selected from table from merchantcenter and from PortalConfig
#----------------------------------------------------------------------------------------
# input        $Servlet | portalConfig object | object
#========================================================================================
sub RemoveProducts {
    my $self = shift;
    my ($Servlet) = @_;

    my $GoogleSmartShoppingPortalConfig = $Servlet->object();

    if ($GoogleSmartShoppingPortalConfig->get('ExportAllProducts')) {
        return;
    }

    my $Shop = $GoogleSmartShoppingPortalConfig->getSite();

    my $aServletProducts = [$Servlet->form->value('ListObjectID')];
    LogGoogleSmartShopping('RemoveProducts - Products marked for deletion', {aServletProducts => $aServletProducts});

    my $GooglePortalService = DE_EPAGES::GoogleSmartShopping::API::GooglePortalService->new({'Shop' => $Shop });
    my $aProducts = $GooglePortalService->getProductAliasByGoogleID($aServletProducts);

    eval {
        $self->_removeProductsInMerchantCenter($Shop, $aServletProducts);
        $GooglePortalService->deleteProductsInPortalConfig($aProducts);
        $Servlet->vars('DeleteSuccessful', 1);
    };

    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping("RemoveProducts failed: ", {
            ErrorMessage => $Error->message(),
        });
    }

    return;
}

#==============================================================================
# Event handlers
#==============================================================================
sub ViewGoogleSmartShoppingProductFeed {
    my $self = shift;
    my ($Servlet) = @_;

    my $GoogleSmartShoppingPortalConfig = $Servlet->object();
    my $Shop = $GoogleSmartShoppingPortalConfig->getSite();

    if ($Shop->isUnity()) {
        $Servlet->vars('ViewAction', 'UnityMBO-ViewGoogleSmartShoppingAddCampaigns');
        $Servlet->vars('TemplateName', 'TabPage-GoogleSmartShoppingProductFeed');
    }

    $Servlet->vars('PortalConfigID', $GoogleSmartShoppingPortalConfig->id());
    my $PortalSiteLanguage = GetInfoLanguage($GoogleSmartShoppingPortalConfig->get('LanguageID'));
    $Servlet->vars('PortalSiteLanguageCode3', $PortalSiteLanguage->{'Code3'});

    my $IsAllowedProductUpload = $self->_areProductUploadRequirementsMet($Shop);
    $Servlet->vars('IsAllowedProductUpload', $IsAllowedProductUpload);
    unless (defined $IsAllowedProductUpload) {
        LogGoogleSmartShopping('UnityViewGoogleSmartShoppingProductFeed - Product Upload not allowed', {IsAllowedProductUpload => 0});
        return $self->SUPER::View($Servlet);
    }

    my $hProductStatuses = $self->_getProductStatistics($Shop);
    my $hPortalConfig = $GoogleSmartShoppingPortalConfig->get([ qw(Products  ExportAllProducts) ]);

    $Servlet->vars('ProductsInPortalConfig', scalar(@{$hPortalConfig->{'Products'}}));
    $Servlet->vars('ProductStatuses', $hProductStatuses);
    $Servlet->vars('ExportAllProducts', $hPortalConfig->{'ExportAllProducts'});

    return $self->_viewPagedChildren($Servlet);
}

sub _areProductUploadRequirementsMet {
    my $self = shift;
    my ($Shop) = @_;

    return unless (defined($Shop->get('Address')->get('Country')));
    return unless ($Shop->existsChild('GoogleSmartShoppingConfig'));
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');

    return unless ($GoogleSmartShoppingConfig->areGoogleMerchantCenterRequirementsMet());
    # most important is that we have Merchant Center account connected
    return unless (isValid($GoogleSmartShoppingConfig->get('MerchantCenterAccountID')));

    return 1;
}

sub _viewPagedChildren {
    my $self = shift;
    my ($Servlet) = @_;

    my $PortalConfig = $Servlet->object();
    my $Shop = $PortalConfig->getSite();
    my $GoogleApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});

    my $Pager = DE_EPAGES::GoogleSmartShopping::API::Object::PagerMerchantCenterProducts->new({
        'Servlet' => $Servlet,
        'Object'  => $PortalConfig,
        'GoogleApiService' => $GoogleApiService
    });

    $Pager->execute();
    return $Pager->page;
}

sub _removeProductsInMerchantCenter {
    my $self = shift;
    my ($Shop, $aProducts) = @_;

    LogGoogleSmartShopping("Remove selected productsIDs ", {
        Products => $aProducts,
    });

    if (scalar @$aProducts > 0) {
        my $GoogleApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});

        return $GoogleApiService->deleteProducts($aProducts);
    }

    return;
}

#========================================================================================
# function     RemoveAllProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       RemoveAllProducts( $Servlet );
#----------------------------------------------------------------------------------------
# description  Remove all products from associated merchant center account.
#----------------------------------------------------------------------------------------
# input        $Servlet | Presentation Object | object
#========================================================================================
sub RemoveAllProducts {
    my $self = shift;
    my ($Servlet) = @_;
    my $PortalConfig = $Servlet->object();
    $PortalConfig->set({'ExportAllProducts', 0});
    $Servlet->vars('ExportAllProducts', 0);

    my $GoogleSmartShoppingPortalConfig = $Servlet->object();
    $GoogleSmartShoppingPortalConfig->removeAllProducts();
    $GoogleSmartShoppingPortalConfig->set({'ExportAllProducts' => 0});

    my $Shop = $GoogleSmartShoppingPortalConfig->getSite();
    my $aGoogleProducts = $self->_collectAllProductsByBatch($Shop);
    LogGoogleSmartShopping('Remove all assigned products in the shop:', {
        "itemCount" => scalar @$aGoogleProducts
    });

    my $DeleteByBatchResult = $self->_deleteProductsByBatch($Shop, $aGoogleProducts);
    $Servlet->vars('DeleteSuccessful', $DeleteByBatchResult);

    return;
}

#========================================================================================
# function     UploadAllProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       UploadAllProducts( $Servlet );
#----------------------------------------------------------------------------------------
# description  Upload all shop products. It also sets the flag to export all products of
#               the Shop to Google.
#----------------------------------------------------------------------------------------
# input        $Servlet | Presentation Object | object
#========================================================================================
sub UploadAllProducts {
    my $self = shift;
    my ($Servlet) = @_;

    my $PortalConfig = $Servlet->object();

    my $aProducts = $PortalConfig->get('Products');
    if (scalar @{$aProducts} > 0)  {
        $PortalConfig->removeAllProducts();
    }

    $PortalConfig->set({'ExportAllProducts', 1});

    return $self->_uploadProductsCommonTasks($Servlet);
}

#========================================================================================
# function     ExportPortalConfig
# state        public
#----------------------------------------------------------------------------------------
# syntax       ExportPortalConfig( $Servlet );
#----------------------------------------------------------------------------------------
# description  ExportPortalConfig uses the parent classes to starts the export process
#               that gathers product info and send it to an API
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet Object | object
#========================================================================================
sub ExportPortalConfig {
    my $self    = shift;
    my ($Servlet) = @_;

    LogGoogleSmartShopping("ExportPortalConfig initiated", {});

    my $PortalConfig = $Servlet->object();

    if ($PortalConfig->get('ExportAllProducts'))  {
        $PortalConfig->set({'ExportAllProducts', 0});
    }

    return $self->_uploadProductsCommonTasks($Servlet);
}

#========================================================================================
# function     ReUploadAllProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       ReUploadAllProducts( $Servlet );
#----------------------------------------------------------------------------------------
# description  Reuploads all Products
#----------------------------------------------------------------------------------------
# input        $Servlet
#========================================================================================
sub ReUploadAllProducts {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_uploadProductsCommonTasks($Servlet);
}

sub _uploadProductsCommonTasks {
    my $self = shift;
    my ($Servlet) = @_;

    my $PortalConfig = $Servlet->object();
    unless (isValid($PortalConfig->get('ShippingMethod'))) {
        my $SelectedShippingMethodId = $self->_getSelectedShippingMethodID($Servlet);
        $self->_setShippingMethodOrError($PortalConfig, $SelectedShippingMethodId);
    }

    $self->SUPER::GeneratePortalConfigProductPrettyUnityUrls($Servlet);
    return $self->SUPER::ExportPortalConfig($Servlet, 0);
}

sub _getProductStatistics {
    my $self = shift;
    my ($Shop) = @_;

    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({ Shop => $Shop });
    my $hResult;
    eval {
        $hResult = $Service->getRatedProductsStatistics();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping("Google ProductLevelIssues request failed: ", {
            Error        => $Error,
            ErrorMessage => $Error->message(),
        });
    }
    return $hResult;
}

#========================================================================================
# function     ChangePortalShippingMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       ChangePortalShippingMethod( $Servlet );
#----------------------------------------------------------------------------------------
# description  Change portal config shipping method from select box
#----------------------------------------------------------------------------------------
# input        $Servlet
#========================================================================================
sub ChangePortalShippingMethod {
    my $self = shift;
    my ($Servlet) = @_;

    my $SelectedShippingMethodId = $self->_getSelectedShippingMethodID($Servlet);
    LogGoogleSmartShopping('ChangePortalShippingMethod: New Shipping method from Template', $SelectedShippingMethodId);

    my $PortalConfig = $Servlet->object();
    my $ShippingMethodId = $self->_setShippingMethodOrError($PortalConfig, $SelectedShippingMethodId);

    $Servlet->params('ContentType', 'application/json');
    $Servlet->response->code(HTTP_OK);
    $Servlet->writeContentOutput({'ShippingMethodID' => $ShippingMethodId});
}

sub _getSelectedShippingMethodID {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form();
    my $hPostData = $Form->formValues();

    return $hPostData->{'selectedShippingMethodId'}->[0];
}

sub _setShippingMethodOrError {
    my $self = shift;
    my ($PortalConfig, $ShippingMethodId) = @_;

    unless (isValid($ShippingMethodId)) {
        LogGoogleSmartShopping('_setShippingMethodOrError: Error getting shippingmethod',
            { 'Error' => 'GSS portal config shipping method is null' }
        );
        return;
    }

    my $Shop = $PortalConfig->getSite();
    my $ShippingMethod;
    eval {
        $ShippingMethod = $Shop->loadObject($ShippingMethodId, 'ShippingMethod');
        TestObject('ShippingMethod', $ShippingMethod, 'DE_EPAGES::Shipping::API::Object::ShippingMethod');
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping(
            '_setShippingMethodOrError: Error getting shippingmethod',
            { 'Error' => $Error->message() }
        );
        return;
    }
    $PortalConfig->set({ 'ShippingMethod' => $ShippingMethod });

    return $ShippingMethodId;
}

sub _collectAllProductsByBatch {
    my $self = shift;
    my ($Shop) = @_;

    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({ Shop => $Shop });
    my @GoogleProducts = ();
    my $ProductsCounter = 0;
    my $Page = 1;
    my $ResultsPerPage = AMOUNT_OF_PRODUCTS_PER_BATCH;
    eval {
        my $hProductListResult;
        do {
            $hProductListResult = $Service->productsStatuses($Page, $ResultsPerPage);
            foreach my $hItem (@{$hProductListResult->{items}}) {
                push(@GoogleProducts, $hItem->{productId});
                $ProductsCounter++;
            }
            $Page++;
        } while ( ($ProductsCounter < $hProductListResult->{results}) && (scalar(@{$hProductListResult->{items}}) > 0) );
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Google products list generation (to be deleted), failed: ', {
            ErrorMessage => $Error->message(),
        });
    }

    return \@GoogleProducts;
}

sub _deleteProductsByBatch {
    my $self = shift;
    my ($Shop, $aGoogleProducts) = @_;

    my $ProductBatchService = DE_EPAGES::GoogleSmartShopping::API::ProductBatchService->new($Shop);
    $ProductBatchService->OnStartBatch();
    my $DeleteSuccessful = 1;
    while (scalar(@$aGoogleProducts)) {
        my @GoogleProductBatch = splice(@$aGoogleProducts, 0, AMOUNT_OF_PRODUCTS_PER_BATCH);
        eval {
            $ProductBatchService->deleteProducts(\@GoogleProductBatch);
        };
        if (ExistsError()) {
            my $Error = GetError();
            LogGoogleSmartShopping('Google products batch deletion, failed: ', {
                ErrorMessage => $Error->message(),
            });
            $DeleteSuccessful = 0;
            next;
        }
    }
    return $DeleteSuccessful;
}

1;
