#==================================================================================================
# package      DE_EPAGES::Ogone::UI::Shop
# state        public
# description  UI class
#==================================================================================================
package DE_EPAGES::Ogone::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

use strict;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Object::API::Factory qw ( ExistsObject );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Order::UI::Basket;
use DE_EPAGES::Ogone::API::Payment qw ( InterpretTransactionStatus UpdateTransactionStatus );
use Digest::SHA1 qw (sha1_hex);
use Encode qw ( encode_utf8 );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::Ogone::API::Tools qw ( Ogone_SHA1 );
use DE_EPAGES::Ogone::API::Constants qw ( @SHA_OUT_PARAMETERS );

use constant PERCENT_FACTOR => 100;
use constant UNITY_CANCELLATION_ERROR => 'Canceled';
use constant UNITY_FAILURE_ERROR => 'Failed';

#==================================================================================================
# function     TransactionPaymentName
# state        private
#--------------------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName();
#--------------------------------------------------------------------------------------------------
# description  return Ogone as string. Name of PaymentType for search/view
#                               transactions.
#--------------------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#==================================================================================================
sub TransactionPaymentName {'Ogone'}

#==================================================================================================
# This callback is sent when PM is Amazon checkout.
#==================================================================================================
sub PaymentBacklinkOgoneAmazon {
    my $self = shift;
    my ($Servlet) = @_;

    $self->BlockMonitor($Servlet);

    my $Object      = $Servlet->object;
    my $hFormValues = $Servlet->params();

    my $Shop              = $Object->getSite;
    my $PaymentLineItemID = $hFormValues->{'PID'};

    LogPayment( 'Ogone', "Amazon checkout Backlink parameters for ID $PaymentLineItemID:\n", $hFormValues );

    my $PaymentLineItem = $Shop->loadObject( $PaymentLineItemID, 'LineItemPaymentOgone' );
    my $BasketOrder     = $PaymentLineItem->container->parent;
    my $ObjectID        = $BasketOrder->id;
    my $PaymentMethod   = $PaymentLineItem->get('PaymentMethod');
    my $passphrase
      = $PaymentMethod->get('IsInTestMode')
      ? $PaymentMethod->get('PassphraseTest')
      : $PaymentMethod->get('Passphrase');

    # reset cached language id etc.
    $Servlet->locale(1);
    $Servlet->languageID(1);
    $Servlet->currencyID(1);

    # We check if the callback from Ogone contains the TXTOKEN
    # If it does then it's the first step, and we have to send them back the whole transaction
    #   information
    # If not, then it's the second and last step, we can save the transaction and order information
    #   depending of the result.

    my $TXTOKEN = $hFormValues->{'TXTOKEN'};

    if ($TXTOKEN) {

        $Servlet->vars( 'FormError', '1' );
        $Servlet->vars( 'Reason',    'first request' );

        #View action for the confirmation page
        $Servlet->vars( 'ViewAction', 'ViewAmazonConfirmation' );
        $Servlet->vars( 'ObjectID',   $ObjectID );

        # transaction feedback for steps One
        # payment parameters

        my $OrderShipCost = $BasketOrder->get('LineItemContainer')->get('Shipping')->get('LineItemPrice');

        if ( defined ($OrderShipCost) ) {
            $OrderShipCost *= PERCENT_FACTOR;
        }
        else {
            $OrderShipCost = 0;
        }

        # if the price model is gross we send 0, if it's net we send the value of the tax
        my $OrderShipTax = 0;

        if ( not $BasketOrder->get('LineItemContainer')->get('TaxModel') ) {
            $OrderShipTax = $BasketOrder->get('LineItemContainer')->get('TotalTax') * PERCENT_FACTOR;
        }

        my %Params = (
            'PSPID' => $PaymentMethod->get('IsInTestMode') ? $PaymentMethod->get('MerchantNameTest')
            : $PaymentMethod->get('MerchantName'),
            'ORDERID'       => $hFormValues->{'ORDERID'},
            'CURRENCY'      => $PaymentLineItem->get('CurrencyID'),
            'LANGUAGE'      => $BasketOrder->get('LineItemContainer')->get('LocaleID'),
            'PM'            => $hFormValues->{'PM'},
            'ORDERSHIPCOST' => $OrderShipCost,
            'ORDERSHIPTAX'  => $OrderShipTax,
            'PAYID'         => $BasketOrder->get('Alias'),
            'CN'            => defined ( $BasketOrder->get('ShippingAddress') )
                ? $BasketOrder->get('ShippingAddress')->get('FullName')
                : $BasketOrder->get('BillingAddress')->get('FullName'),
            'OWNERADDRESS' => defined ( $BasketOrder->get('ShippingAddress') )
                ? $BasketOrder->get('ShippingAddress')->get('Street')
                : $BasketOrder->get('BillingAddress')->get('Street'),
            'OWNERTOWN' => defined ( $BasketOrder->get('ShippingAddress') )
                ? $BasketOrder->get('ShippingAddress')->get('City')
                : $BasketOrder->get('BillingAddress')->get('City'),
            'OWNERZIP' => defined ( $BasketOrder->get('ShippingAddress') )
                ? $BasketOrder->get('ShippingAddress')->get('Zipcode')
                : $BasketOrder->get('BillingAddress')->get('Zipcode'),
            'OWNERCTY' => defined ( $BasketOrder->get('ShippingAddress') )
                ? $BasketOrder->get('ShippingAddress')->get('City')
                : $BasketOrder->get('BillingAddress')->get('City'),
            'TXTOKEN' => $hFormValues->{'TXTOKEN'},
            'ORIG'    => 'ePages',
        );

        #Check the item in the basket
        #Get the list of items in the basket
        my $LineItems = [];

        $LineItems = $BasketOrder->get('LineItemContainer')->get('LineItems');
        my $ItemNumber = 1;

        if ( scalar ( @{$LineItems} ) == 1 ) {

            # we have only one item or more of the same product, we dont iterate
            $Params{'ITEMID1'}    = $ItemNumber;
            $Params{'ITEMNAME1'}  = @{$LineItems}[0]->get('Name');
            $Params{'ITEMPRICE1'} = @{$LineItems}[0]->get('BasePrice');
            $Params{'ITEMQUANT1'} = @{$LineItems}[0]->get('Quantity');

        }
        else {
            foreach my $Item (@$LineItems) {
                $Params{ 'ITEMID' . $ItemNumber }    = $Item->get('ID');
                $Params{ 'ITEMNAME' . $ItemNumber }  = $Item->get('Name');
                $Params{ 'ITEMPRICE' . $ItemNumber } = $Item->get('BasePrice');
                $Params{ 'ITEMQUANT' . $ItemNumber } = $Item->get('Quantity');

                $ItemNumber++;
            }
        }

        $Params{'AMOUNT'} = $BasketOrder->get('LineItemContainer')->get('GrandTotal') * PERCENT_FACTOR;

        $Params{'ACCEPTURL'} = BuildShopUrl(
            $Shop,
            {
                'ChangeAction' => 'PaymentBacklinkOgoneAmazon',
                'PID'          => $PaymentLineItemID,
                'RET'          => 'SUCCESS'
            },
            {
                'UseSSL'        => 1,
                'UseObjectPath' => 0
            }
        );

        $Params{'DECLINEURL'} = BuildShopUrl(
            $Shop,
            {
                'ChangeAction' => 'PaymentBacklinkOgone',
                'PID'          => $PaymentLineItemID,
                'RET'          => 'DECLINED'
            },
            {
                'UseSSL'        => 1,
                'UseObjectPath' => 0
            }
        );

        $Params{'EXCEPTIONURL'} = BuildShopUrl(
            $Shop,
            {
                'ChangeAction' => 'PaymentBacklinkOgone',
                'PID'          => $PaymentLineItemID,
                'RET'          => 'EXCEPTION'
            },
            {
                'UseSSL'        => 1,
                'UseObjectPath' => 0
            }
        );

        $Params{'CANCELURL'} = BuildShopUrl(
            $Shop,
            {
                'ChangeAction' => 'PaymentBacklinkOgone',
                'PID'          => $PaymentLineItemID,
                'RET'          => 'CANCELED'
            },
            {
                'UseSSL'        => 1,
                'UseObjectPath' => 0
            }
        );

        $Params{'BACKURL'} = $BasketOrder->getSite()->get('WebUrl');

        $Params{'HOMEURL'} = $BasketOrder->getSite()->get('WebUrl');

        # Sort the hash elements
        my $digest = '';

        foreach my $key ( sort _byNumber keys %Params ) {

            # now with the list ordered we remove the zeros in the string between text and number

            if ( $key =~ /^([A-Z]+)([0-9]+)$/x ) {

                # remove zeros from $2 and concatenate with $1
                my $text   = $1;
                my $number = $2;
                $number =~ s/^0+//;
                $key = $text . $number;
            }

            $digest = $digest . $key . '=' . $Params{$key} . $passphrase;
        }

        # calculate hash
        $Params{'SHASIGN'} = uc ( sha1_hex( encode_utf8($digest) ) );

        # debugging
        LogPayment(
            'Ogone',
            'params for payment line item with Amazon Checkout, first step'
              . $PaymentLineItem->get('GUID')
              . 'to send to url',
            \%Params
        );

        $Servlet->vars( 'AmazonCheckoutParams', [ map { { 'Name' => $_, 'Value' => $Params{$_} } } keys %Params ] );

    }
    else {

        # although it's only called on success, we test the parameters and show errors.
        my $hValues = $self->_ProcessPaymentParameters( $Servlet, $hFormValues, $PaymentLineItem, 1 );

        if (defined $hValues->{'ViewObject'}) {
            my $CustomerOrder = $hValues->{'ViewObject'};
            $Servlet->vars( 'ViewObjectID', $CustomerOrder->id );
            $Servlet->vars( 'ViewAction',   'View' );
        }
    }
    return;
}

#==================================================================================================
# function     _byNumber
# state        private
#--------------------------------------------------------------------------------------------------
# syntax       _byNumber $a $b
#--------------------------------------------------------------------------------------------------
# description  This function is applied when call the function sort to compare the strings that
#               have numbers at the end
#--------------------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#==================================================================================================
sub _byNumber {

    my $locA = $a;
    $locA =~ s/\d+$/sprintf("%03d", $&)/e;
    my $locB = $b;
    $locB =~ s/\d+$/sprintf("%03d", $&)/e;

    # Compare as strings
    $locA cmp $locB;
}

#==================================================================================================
# Delayed callback that update the status of the transaction done with Amazon Checkout
#==================================================================================================
sub PaymentCallbackOgoneAmazonStatus {

    my $self = shift;
    my ($Servlet) = @_;

    $self->BlockMonitor($Servlet);

    my $hFormValues = $Servlet->params();
    my $Shop = $Servlet->object();

    LogPayment( 'Ogone', 'Callback amazon checkout status', $hFormValues );

    # Match and capture the value before the underscore: 27432_1688565791803.1
    my $BasketOrOrderId = ($hFormValues->{'orderID'} =~ /^([^_]+)/)[0];

    my $Order = $self->_loadCustomerOrder($Shop, $BasketOrOrderId);

    my $PaymentLineItem = $Order->get('LineItemContainer')->get('Payment');

    # calling InterpretTransactionStatus
    my $Status = InterpretTransactionStatus($PaymentLineItem, $hFormValues->{'STATUS'}, $hFormValues->{'NCERROR'});
    my $hTransactionStatus = $self->_getTransactionStatus($PaymentLineItem, $Status);
    $self->_setOrderStatus($Order, $hTransactionStatus, $Status);

    return $hTransactionStatus;
}

sub _loadCustomerOrder {
    my $self = shift;
    my ($Shop, $ObjectId) = @_;

    my $Order;

    eval {
        $Order = $Shop->loadObject($ObjectId, 'CustomerOrder');
    };
    if(ExistsError()) {
        $Order = $self->_loadCustomerOrderFromBasket($Shop, $ObjectId);
    };

    return $Order;
}

sub _loadCustomerOrderFromBasket {
    my $self = shift;
    my ($Shop, $ObjectId) = @_;

    my $Order;

    eval {
        my $Basket = $Shop->loadObject($ObjectId, 'Basket');
        my $OrderGuid = $Basket->get('OrderGUID');
        $Order = $Shop->loadObjectByGuid($OrderGuid, 'CustomerOrder');
    };
    if(ExistsError()) {
        Error('Invalid basket or order ID: ' . $ObjectId . ' for Shop ' . $Shop->alias);
    };

    return $Order;
}

#========================================================================================
# function     $hTransactionStatus = _getTransactionStatus
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hTransactionStatus = _getTransactionStatus( $PaymentLineItem, $Status );
#----------------------------------------------------------------------------------------
# description  it provides a hash with the transaction status information
#----------------------------------------------------------------------------------------
# input        $PaymentLineItem | Payment line item object | object
# input        $Status | Status srting | String
# return       $hTransactionStatus | transaction status and timing hash | refhash
#========================================================================================
sub _getTransactionStatus {
    my $self = shift;
    my ($PaymentLineItem, $Status) = @_;

    my $hTransactionStatus;

    if ( $Status eq 'OK' ) {
        $hTransactionStatus = UpdateTransactionStatus( $PaymentLineItem, 'STATUS_PAID' );
    } elsif ( $Status eq 'KO' ) {
        $hTransactionStatus = UpdateTransactionStatus( $PaymentLineItem, 'STATUS_FAILED' );
    } elsif ( $Status eq 'Wait') {
        $hTransactionStatus = UpdateTransactionStatus( $PaymentLineItem, 'STATUS_PENDING' );
    }

    return $hTransactionStatus;
}

#========================================================================================
# function     $hTransactionStatus = _setTransactionStatus
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hTransactionStatus = _setTransactionStatus( $Order, $hTransactionStatus, $Status );
#----------------------------------------------------------------------------------------
# description  it sets/changes transaction status and timing of the change
#----------------------------------------------------------------------------------------
# input        $Order | Order object | object
# input        $hTransactionStatus | Transaction status hash  | hash.ref
# input        $Status | Status string | String
#========================================================================================
sub _setOrderStatus {
    my $self = shift;
    my ($Order, $hTransactionStatus, $Status) = @_;

    return unless $Order->instanceOf('CustomerOrder');

    if ( $Status eq 'OK' && !isValid($Order->get('PaidOn'))) {
        $Order->set( { 'PaidOn' => $hTransactionStatus->{'TransTime'} } );
        $Order->set( { 'PendingOn' => undef } );
    }
}

#==================================================================================================
# This callback is sent to the success link.
#==================================================================================================
sub PaymentBacklinkOgone {
    my $self = shift;
    my ($Servlet) = @_;

    $self->BlockMonitor($Servlet);

    my $hFormValues = $Servlet->params();
    LogPayment( 'Ogone', 'Backlink parameters', $hFormValues );

    my $PaymentLineItemID = $hFormValues->{'PID'};

    # reset cached language id etc.
    $Servlet->locale(1);
    $Servlet->languageID(1);
    $Servlet->currencyID(1);

    my $Form = $Servlet->form();

    # check if PaymentLineItemID exists.
    $Form->executeFormError( {
            'Name'       => 'GUID',
            'Reason'     => 'BadCallback_NoPaymentGUID',
            'ViewAction' => 'ViewPaymentOgoneResult',
        }
    ) unless ( isValid($PaymentLineItemID) && ExistsObject($PaymentLineItemID) );

    my $Shop = $Servlet->object->getSite();

    my $PaymentLineItem = $Shop->loadObject( $PaymentLineItemID, 'LineItemPaymentOgone' );
    my $Object = $PaymentLineItem->container()->parent();

    # test the parameters of Ogone, show errors if something is wrong
    my $hValues = $self->_ProcessPaymentParameters( $Servlet, $hFormValues, $PaymentLineItem, 1 );

    my $ErrorReason;

    if ( $hFormValues->{'RET'} eq 'CANCELED' ) {
        $ErrorReason = 'PaymentProcess_Canceled';
    } elsif ( $hFormValues->{'RET'} eq 'ERROR' ) {
        $ErrorReason = 'PaymentProcess_Failed';
    } elsif ( $hFormValues->{'RET'} eq 'DECLINED' ) {
        $ErrorReason = 'PaymentProcess_Declined';
    } elsif ( $hFormValues->{'RET'} eq 'EXCEPTION' ) {
        $ErrorReason = 'PaymentProcess_Exception';
    }

    if (defined $ErrorReason) {
        my $ViewAction =  'ViewPaymentOgoneResult';

        if ($Shop->isUnity()) {
            $PaymentLineItem->set( { 'PaymentErrorCode' => UNITY_CANCELLATION_ERROR } );
            return $self->_redirectAfterUnityPayment($Servlet, $Object, $ErrorReason);
        }

        $Form->executeFormError({
            'Name'        => 'GUID',
            'Reason'      => $ErrorReason,
            'ViewAction'  => $ViewAction,
            'ObjectID'    => $Object->id,
            'ObjectClass' => $Object->class->alias(),
        });
    }

    # show customerorder on success (and Now failures)
    if ( defined $hValues->{'ViewObject'} ) {
        my $CustomerOrder = $hValues->{'ViewObject'};
        $Servlet->vars( 'ViewObjectID', $CustomerOrder->id );
        $Servlet->registerSession( $CustomerOrder->get('User'), 0 );

        if ($Shop->isUnity()) {
            if ($hFormValues->{'RET'} eq 'SUCCESS') {
                $PaymentLineItem->set( { 'PaymentErrorCode' => undef } );
            }
            return $self->_redirectAfterUnityPayment($Servlet, $Object);
        }
    }

    return;
}

#==================================================================================================
sub PaymentCallbackOgone {
    my $self = shift;
    my ($Servlet) = @_;

    my $hFormValues = $Servlet->params;
    my $Shop = $Servlet->object;
    LogPayment( 'Ogone', 'Callback parameters', $hFormValues );

    # Match and capture the value before the underscore: 27432_1688565791803.1
    my $OrderID = ($hFormValues->{'orderID'} =~ /^([^_]+)/)[0];

    my $Order;
    eval {
        $Order = $Shop->loadObject($OrderID, 'CustomerOrder');
    };
    if (ExistsError()) {
        # it might be a "Basket" object from OrderedBaskets list
        my $Error = GetError();
        if ( $Error->code eq 'NoResults_ObjectID' || $Error->code eq 'ClassMismatch' ) {
            $Order = $Shop->loadObject($OrderID, 'Basket');
        }
    }
    Error( 'Invalid order/basket ID: ' . $OrderID . ' for Shop ' . $Shop->alias) unless (defined $Order);

    my $LineItemContainer = $Order->get('LineItemContainer');
    my $PaymentLineItem   = $LineItemContainer->get('Payment');

    # test the parameters, show errors if something is wrong.
    # Last boolean paremeter specifies if call comes from Backlink (1=>use error forms) or CallBack (0=>use log errors)
    $self->_ProcessPaymentParameters( $Servlet, $hFormValues, $PaymentLineItem, 0 );

    # Send a HTTP OK
    $Servlet->params( 'ContentType',    'text/plain' );
    $Servlet->params( 'ExitAfterEvent', 1 );
    $Servlet->writeContentOutput('OK');

    return;
}

#==================================================================================================
sub _ProcessPaymentParameters {
    my $self = shift;
    my ( $Servlet, $hFormValues, $PaymentLineItem, $isBackLink ) = @_;

    my $Shop            = $Servlet->object;
    my $Form            = $Servlet->form;
    my $BasketOrder     = $PaymentLineItem->container->parent;
    my $strIsBacking = $isBackLink ? 'BackLink' : 'CallBack';

    my $ErrorViewAction = 'ViewPaymentOgoneResult';
    my $PaymentMethod   = $PaymentLineItem->get('PaymentMethod');
    my $passphrase
      = $PaymentMethod->get('IsInTestMode') ? $PaymentMethod->get('PassphraseTest') : $PaymentMethod->get('Passphrase');

    # check if GUID belongs to this shop.
    my $PaymentShop = $PaymentLineItem->getSite;

    if ( $PaymentShop->id != $Shop->id ) {
        if ($isBackLink) {
            $PaymentLineItem->set({'PaymentErrorCode' => UNITY_FAILURE_ERROR});

            $Form->executeFormError( {
                    'Name'       => 'GUID',
                    'Reason'     => 'BadCallback_ShopIDMisMatch',
                    'ViewAction' => $ErrorViewAction
                }
            );
        }
        else {
            Error( 'Payment '. $PaymentLineItem->alias . ' belongs to ' . $Shop->alias . ' shop, but it tries to use '
                  . $PaymentShop->alias . ' shop' );
        }
    }

    #check merchant configuration from returned parameters
    _checkMerchantConfiguration( $Servlet, $hFormValues, $ErrorViewAction, $isBackLink, $PaymentLineItem );

    # DO NOT set this directly, instead check callback or
    # communicate with payment to avoid abuse, e.g. by
    # Check the SHA-1-OUT
    # calculate hash
    my $SHA1SignOut = Ogone_SHA1( $passphrase, $hFormValues, \@SHA_OUT_PARAMETERS );

    # initialize return hash
    my $hTransactionStatus;

    if ( $SHA1SignOut eq $hFormValues->{'SHASIGN'} ) {

        # calling InterpretTransactionStatus
        my $payStatus = InterpretTransactionStatus( $PaymentLineItem, $hFormValues->{'STATUS'}, $hFormValues->{'NCERROR'} );

        if ( isValid($payStatus) ) {
            if ( $payStatus eq 'Repeated' ) {
                LogPayment( 'Ogone', "$strIsBacking: Repeated call for already existing OrderID: ",
                            $hFormValues->{'PAYID'} );
                return;
            }
            $hTransactionStatus = $self->_getTransactionStatus($PaymentLineItem, $payStatus);
        }
        else {
            # we don't know how to process the returned status
            LogPayment('Ogone', "$strIsBacking: Unknown  return status: ", $hFormValues->{'STATUS'}
            );
            if (!$isBackLink) {
                # backlink errors are catched through 'RET' returned parameter
                Error( 'Unknown  return status: ' . $hFormValues->{'STATUS'} );
            }
        }

        if ( $BasketOrder->instanceOf('Basket') and ($hTransactionStatus->{'TransStatus'} ne 'STATUS_FAILED') ) {
            # successful return to basket and no order exists => create order
            $BasketOrder = DE_EPAGES::Order::UI::Basket->doBasket2Order( $Servlet, $BasketOrder );
        }

        $self->_setOrderStatus($BasketOrder, $hTransactionStatus, $payStatus);

        if ( $isBackLink ) {
            # show confirmation page for Base on success and Now (either success or fail)
            $hTransactionStatus->{'ViewObject'} = $BasketOrder;
        } else {
            $hTransactionStatus->{'ViewObject'} = undef;
        }

        # used for debugging
        LogPayment( 'Ogone', "$strIsBacking: Ogone transaction Params for PaymentLineItem", $hTransactionStatus );

    }
    else {
        # If the SHA-1-OUT doesnt match the one sent by Ogone
        # used for debugging
        LogPayment('Ogone',
                    "$strIsBacking: Mismatch between Ogone returned SHASIGN parameter and our generated signature: ",
                    $SHA1SignOut);
        if ($isBackLink) {
            $PaymentLineItem->set({'PaymentErrorCode' => UNITY_FAILURE_ERROR});

            $Form->executeFormError( {
                    'Name'       => 'GUID',
                    'Reason'     => 'BadCallback_CallbackPWMisMatch',
                    'ViewAction' => $ErrorViewAction,
                }
            );
        }
        else {
            Error('SHASIGN Mismatch');
        }
    }

    return $hTransactionStatus;
}

#==============================================================================
sub ViewPaymentError {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::View($Servlet);

    my $Shop    = $Servlet->object();
    my $hParams = $Servlet->params();

    #LineItemPaymentOgone malformed check is done into PaymentBacklinkOgone function (Reason: BadCallback_NoPaymentGUID)
    my $LineItemPaymentOgone;
    eval { $LineItemPaymentOgone = $Shop->loadObject( $hParams->{'PID'}, 'LineItemPaymentOgone' ); };
    if ( !ExistsError() ) {
        $Servlet->vars( 'PaymentType', $LineItemPaymentOgone->get('PaymentType')->tleHash );
    }
    else {
        $Servlet->vars( 'PaymentType', { 'Alias' => '??' } );
    }

    return;
}

#==============================================================================
sub _checkMerchantConfiguration {
    my ( $Servlet, $hFormValues, $ErrorViewAction, $isBackLink, $PaymentLineItem ) = @_;

    my $Shop = $Servlet->object;
    my $Form = $Servlet->form;

    #Check that we received transaction feedback by checking signature is sent back.
    #Otherwise it means Merchant's account is not properly configured.
    if ( !isValid( $hFormValues->{'SHASIGN'} ) ) {
        LogPayment(
            'Ogone',
            'Merchant\'s account not properly configured for Shop',
            { 'ShopID' => $Shop->id, 'ShopPath' => $Shop->pathString }
        );
        if ($isBackLink) {
            $PaymentLineItem->set( { 'PaymentErrorCode' => UNITY_FAILURE_ERROR } );

            $Form->executeFormError( {
                    'Name'       => 'GUID',
                    'Reason'     => 'BadCallback_AccountMisconf',
                    'ViewAction' => $ErrorViewAction,
                }
            );
        }
        else {
            Error(  'SHASIGN parameter was empty. Payment administration account misconfigured. ShopID: '
                  . $Shop->id
                  . '  ShopPath: '
                  . $Shop->pathString );
        }
    }

    return;
}

sub _redirectAfterUnityPayment {
    my $self = shift;
    my ($Servlet, $BasketOrOrder, $ErrorCode) = @_;

    my $ViewAction = $BasketOrOrder->instanceOf('CustomerOrder') ? 'ViewMultiCheckoutCustomerOrder' : 'ViewMultiCheckoutPayment';
    my $BasketOrOrderID = $BasketOrOrder->id();

    my $hParams = {
        'ViewAction' => $ViewAction,
        'ErrorPaymentOgone' => $ErrorCode,
        'ObjectID' => $BasketOrOrderID,
    };

    my $hOptions = {
        'Type' => 'sf',
        'UseSSL' => 1,
        'AddToken' => 1,
    };
    my $Url = BuildShopUrl($BasketOrOrder, $hParams, $hOptions, $Servlet);

     LogPayment(
        'Ogone',
        'Ogone redirect to Now after payment',
        { 'hParams' => $hParams, 'BasketOrOrderID' => $BasketOrOrderID }
     );

    $Servlet->redirect($Url);
    return;
}

1;
