#========================================================================================
# package      DE_EPAGES::Order::API::Object::CustomerOrder
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for CustomerOrder.
#========================================================================================
package DE_EPAGES::Order::API::Object::CustomerOrder;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( Compare );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject LoadObjectByPath );
use DE_EPAGES::Order::API::Constants qw ( ORDER_STATUS_ATTRIBUTES );
use DE_EPAGES::Order::API::Order qw ( SendStatusChangeMail );
use DE_EPAGES::Permission::API::Table::Permission qw ( InsertPermission );
use DE_EPAGES::Shop::API::Address qw (GetTaxCountryIDForShop IsEUAddress);
use DE_EPAGES::Tax::API::Shop qw( TaxMatrixByEUAddress );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $CustomerOrder->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'CustomerOrder' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $CustomerOrder->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $class->insert($hInfo);
# example      DE_EPAGES::Order::API::Object::CustomerOrder->insert($hInfo);
# example      my $Class = LoadClassByAlias('CustomerOrder');
#               my $CustomerOrder = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert order in to database (adds permission for user to view old order), afterwards the hook 'OBJ_InsertCustomerOrder' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $CustomerOrder | new object | object
# hook         OBJ_InsertCustomerOrder | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>OrderID       - object identifier    - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my %Info = %$hInfo;
    my $self = $class->SUPER::insert(\%Info);

    if (defined $hInfo->{'User'}) {
        my $Role = LoadObjectByPath('/Classes/CustomerOrder/Roles/Customer');
        InsertPermission({'ObjectID'=>$self->id,
                          'TrusteeID'=>$hInfo->{'User'}->id,
                          'MetaActionID'=>$Role->id,
                          'Allow'=>1 });
    }
    TriggerHook('OBJ_InsertCustomerOrder', {'Object' => $self, 'OrderID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Shop Customer User LineItemContainer ShopAddress BillingAddress ShippingAddress ViewedOn CancelledOn InProcessOn PendingOn ReadyForShippingOn PartlyDispatchedOn DispatchedOn ShippedOn PartlyInvoicedOn InvoicedOn PartlyPaidOn PaidOn ClosedOn ArchivedOn ReturnedOn ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $CustomerOrder->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteCustomerOrder' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteCustomerOrder | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>OrderID       - object identifier    - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteCustomerOrder', {'Object' => $self, 'OrderID' => $self->id });
    $self->set({
        'User' => undef,
        'LineItemContainer' => undef,
        'ShopAddress' => undef,
        'BillingAddress' => undef,
        'ShippingAddress' => undef,
    });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $CustomerOrder->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateCustomerOrder' and 'OBJ_AfterUpdateCustomerOrder'.
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateCustomerOrder | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>OrderID       - object identifier    - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateCustomerOrder | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>OrderID       - object identifier    - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#----------------------------------------------------------------------------------------
# input        $hValues | new attribute values for the given object | ref.hash
# input        $LanguageID | language identifier (optional) | int
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    my $Order = $self;
    my @SetStatusAttributes = grep { exists $hValues->{$_} } ORDER_STATUS_ATTRIBUTES;
    my $HasStatusChanged = scalar @SetStatusAttributes ? 1 : 0;
    my $hOldStatus = $HasStatusChanged ? $Order->get( \@SetStatusAttributes ) : undef;

    TriggerHook('OBJ_BeforeUpdateCustomerOrder', {'Object'=>$self, 'Values'=> $hValues, 'OrderID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateCustomerOrder',  {'Object'=>$self, 'Values'=> $hValues, 'OrderID' => $self->id, 'LanguageID' => $LanguageID});

    # only for changes to an original order (that's not in editing mode)
    if (!$Order->get('OriginalOrder') && $HasStatusChanged) {

        # send the status change mail if the status attribute is set, but was not set before
        foreach my $StatAtt ( @SetStatusAttributes ){
            if ((not defined $hOldStatus->{$StatAtt}) and $hValues->{$StatAtt}){
                eval { SendStatusChangeMail($Order,$StatAtt); };
                if (ExistsError()) {
                    my $Error = GetError();
                    LogError($Error);
                    Error('SEND_MAIL_FAILED', $Error->vars(), $Error);
                }
            }
        }

        # record a change to the LastUpdate profile CustomerOrder.Status if the status or date has changed
        if (grep { Compare('DateTime', $hOldStatus->{$_}, $hValues->{$_}) != 0 } @SetStatusAttributes ) {
            my $StatusProfile = LoadClassByAlias('CustomerOrder')->lastUpdateProfile('Status');
            $self->setUpdated($StatusProfile);
        }
    }

    my $OrderChangeProfile = LoadClassByAlias('CustomerOrder')->lastUpdateProfile('OrderChange');
    $self->setUpdated($OrderChangeProfile);

    return;
}

#========================================================================================
# function     addLineItemContainer
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Order->addLineItemContainer( $hValues );
# example      $Order->addLineItemContainer( { 'CurrencyID' => 'USD', 'LocaleID' => 'en_US' } );
#----------------------------------------------------------------------------------------
# description  Adds a line item container to a new order that does not have a line item
#               container.
#               Note: Do not usde this method directly!
#               Use <function DE_EPAGES::Order::API::Order::CreateOrder> instead.
#----------------------------------------------------------------------------------------
# input        $hValues | <ul>
#                   <li>CurrencyID - (optional; default: user currency or shop default currency) alpha currency code (ISO 4217) - char(3)
#                   <li>LocaleID - (optional; default: user locale or shop default locale) locale id - string
#                   <li>TaxArea - (optional; default: customer tax area) tax area - DE_EPAGES::Tax::API::Object::TaxArea
#                   <li>TaxModel - (optional; default: customer tax model) gross (1) or net (0) price  - int
#                   <li>Class - (optional; default: LineItemContainer) use custom LineItemContainer class - string
#               </ul> | ref.hash (optional)
# return       $Order | new customer order | object
#========================================================================================
sub addLineItemContainer {
    my $self = shift;
    my ($hValues) = @_;

    my $Customer = $self->get('Customer');
    my $User = $self->get('User');
    my $Shop = $self->getSite();

    my $CurrencyID = $hValues->{'CurrencyID'} // (defined $User ? $User->get('CurrencyID') : $Shop->get('CurrencyID'));
    my $LocaleID = $hValues->{'LocaleID'} // (defined $User ? $User->get('LocaleID') : $Shop->get('LocaleID'));


    # TODO use correct tax area here

    my $TaxArea = $hValues->{'TaxArea'} // $Customer->get('TaxArea');
    my $TaxModel = $hValues->{'TaxModel'} // $Customer->get('TaxModel');

    my $Container = $self->insertChild($hValues->{'Class'} // 'LineItemContainer', {
        'Alias'         => 'LineItemContainer',
        'LocaleID'      => $LocaleID,
        'CurrencyID'    => $CurrencyID,
        'TaxArea'       => $TaxArea,
        'TaxModel'      => $TaxModel,
    });
    $self->set({ 'LineItemContainer' => $Container });

    return $Container;
}

#========================================================================================
# function     importXML
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = __PACKAGE__->importXML($Handler, $hElement, $Class, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Imports an object from an XML file. This function is called at the XML
#               start tag. The end tag triggers the <function importXMLEnd> function.
#----------------------------------------------------------------------------------------
# input        $Handler | xml import handler with support function addObject | object
# input        $hElement | XML element structure, see XML::SAX | ref.hash
# input        $Class | class object; the new object will be an instance of this class | object
# input        $hAttributes | attributes hash | ref.hash
# return       $Object | the imported object | object
#========================================================================================
sub importXML {
    my $class = shift;
    my ($Handler, $hElement, $Class, $hAttributes) = @_;

    my $XMLParentObject = $Handler->topID('Object');
    # if reference of object
    if (not $Handler->topID('IsReference')) {
        my $Parent;
        if (defined $hAttributes->{'Parent'}) {
            $Parent = blessed $hAttributes->{'Parent'} ? $hAttributes->{'Parent'} : $XMLParentObject->path($hAttributes->{'Parent'});
        } else {
            $Parent = $XMLParentObject;
        }

        # get the customer
        $hAttributes->{'Customer'} = $Parent->instanceOf('Customer') ?  $Parent :  $Parent->parentByClass('Customer') unless defined $hAttributes->{'Customer'};
        $hAttributes->{'Customer'} = $Handler->topID('Customer') unless defined $hAttributes->{'Customer'};

        my $OrderOrderPath = $hAttributes->{'Path'};
        if (not defined $hAttributes->{'Customer'} and defined $OrderOrderPath){
            my $OrderID = $XMLParentObject->findXML({'Path'=>$OrderOrderPath});
            $hAttributes->{'Customer'} = LoadObject($OrderID)->parent->parent if defined $OrderID;
        }
        Error('NO_Customer', $hAttributes) unless defined $hAttributes->{'Customer'};
        $hAttributes->{'Shop'} = $hAttributes->{'Customer'}->getSite();
    }

    return $class->SUPER::importXML($Handler, $hElement, $Class, $hAttributes );
}

#========================================================================================
# function     importXMLEnd
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = $Object->importXMLEnd($Handler, $hElement);
#----------------------------------------------------------------------------------------
# description  Sets the attribute "LineItemContainer" to the child object with the alias
#               "LineItemContainer" for compatibility with previously exported XML files.
#----------------------------------------------------------------------------------------
# input        $Handler | xml import handler with support function addObject | object
# input        $hElement | XML element structure, see XML::SAX | ref.hash
#========================================================================================
sub importXMLEnd {
    my $self = shift;
    my ($Handler, $hElement) = @_;

    $self->SUPER::importXMLEnd($Handler, $hElement);

    # Add references to child object LineItemContainer.
    # This is only required for compatibility with old export files (pre 6.9.2).
    if( !defined $self->get('LineItemContainer') && $self->existsChild('LineItemContainer') ) {
        $self->set({ 'LineItemContainer' => $self->child('LineItemContainer') });
    }

    return;
}

#========================================================================================
# function     exportXMLAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hAttributes = $Object->exportXMLAttributes( $Driver, $hAllAttributes );
#----------------------------------------------------------------------------------------
# description  Returns plain attributes of object to XML export driver. This function
#               removes bit attributes with value 0.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function exportableAttributes> | ref.hash
# return       $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLAttributes {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;
    $hAttributes = $self->SUPER::exportXMLAttributes($Driver, $hAttributes);

    #delete reference if it is anonymous user
    if (exists $hAttributes->{'User'} and defined $hAttributes->{'User'} ) {
        delete $hAttributes->{'User'} if $self->get('User')->get('IsAnonymous');
    }
    return $hAttributes;
}

#========================================================================================
# function     exportXMLElements
# state        private
#----------------------------------------------------------------------------------------
# syntax       $aElements = $Object->exportXMLElements($Driver, $hAllAttributes);
#----------------------------------------------------------------------------------------
# description  export xml data to export multilinguagal properties of this object.
#----------------------------------------------------------------------------------------
# input        $Driver | export driver | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function allAttributes> | ref.hash
#========================================================================================
sub exportXMLElements {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;

    $self->SUPER::exportXMLElements($Driver, $hAttributes);

    if ($self->class->get('ExportLevel') == $Driver->exportLevel) {
        # add references to child objects
        my %Attributes;
        foreach my $AttributeAlias ( qw( BillingAddress ShippingAddress LineItemContainer ) ) {
            my $Object = $self->get($AttributeAlias);
            $Attributes{$AttributeAlias} = $self->relativePath($Object) if defined $Object;
        }
        if( scalar keys %Attributes ) {
            $Driver->addObjectReference('CustomerOrder', $self, \%Attributes);
        }
    }
    return;
}

#========================================================================================
# function     getExtraExportableAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aAttributes = $Object->getExtraExportableAttributes();
#----------------------------------------------------------------------------------------
# description  Returns names of attributes of object for XML export of orders.
#----------------------------------------------------------------------------------------
# return       $aAttributes | plain attributes | ref.array.string
#========================================================================================
sub getExtraExportableAttributes {
    return [qw(
        CreationDate
        CustomerComment
        ViewedOn
        CancelledOn
        InProcessOn
        PendingOn
        ReadyForShippingOn
        PartlyDispatchedOn
        DispatchedOn
        ShippedOn
        PartlyInvoicedOn
        InvoicedOn
        PartlyPaidOn
        PaidOn
        ClosedOn
        ArchivedOn
        ReturnedOn
    )];
}

#========================================================================================
# function     clone
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Clone = $CustomerOrder->clone( $hInfo );
# syntax       $Clone = $CustomerOrder->clone( $hInfo, $Recursive );
#----------------------------------------------------------------------------------------
# description  Inserts a new order into the database as copy from old order.
#----------------------------------------------------------------------------------------
# input        $hInfo | hash with mandatory attribute values.
#                       <ul>
#                         <li>ClassID - class id - int</li>
#                         <li>Alias - alias - varchar(255)</li>
#                         <li>ParentID - parent objectid - int</li>
#                         <li>Inherit - (optional) inherit permissions from parent object - boolean</li>
#                       </ul> | hash ref
# input        $Recursive | clone child object too (optional, default is false) | boolean
# return       $Clone | the new object | object
#========================================================================================
sub clone {
    my $self = shift;
    my ($hInfo, $Recursive) = @_;
    $Recursive = 0 unless defined $Recursive;

    $hInfo->{'Customer'} = $self->get('Customer') unless exists $hInfo->{'Customer'};
    $hInfo->{'Shop'}     = $self->getSite()     unless exists $hInfo->{'Shop'};

    my $Clone = $self->SUPER::clone($hInfo, $Recursive);

    $Clone->set({'LineItemContainer' => $Clone->child('LineItemContainer')}) if $Clone->existsChild('LineItemContainer');
    $Clone->set({'BillingAddress' => $Clone->child('BillingAddress')}) if $Clone->existsChild('BillingAddress');
    $Clone->set({'ShippingAddress' => $Clone->child('ShippingAddress')}) if $Clone->existsChild('ShippingAddress');
    $Clone->set({'ShopAddress' => $Clone->child('ShopAddress')}) if $Clone->existsChild('ShopAddress');
    return $Clone;
}

#========================================================================================
# function     container
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemContainer = $CustomerOrder->container;
#----------------------------------------------------------------------------------------
# description  Returns the line item container of the order.
#----------------------------------------------------------------------------------------
# return       $LineItemContainer | line item container | DE_EPAGES::Order::API::Object::LineItemContainer
#========================================================================================
sub container {
    my $self = shift;
    return $self->get('LineItemContainer');
}

#========================================================================================
# function     recalculate
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CustomerOrder->recalculate;
#----------------------------------------------------------------------------------------
# description  Recalculate lineitem container, see <function DE_EPAGES::Order::API::Object::LineItemContainer::recalculate>.
#----------------------------------------------------------------------------------------
# input        @Parameter | list of parameters to be changed | array
#========================================================================================
sub recalculate {
    my $self = shift;
    my @Parameter = @_;
    $self->container->recalculate(@Parameter)
}

#========================================================================================
# function     insertLineItem
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CustomerOrder->insertLineItem;
#----------------------------------------------------------------------------------------
# description  Insert a lineItem in to container, see <function DE_EPAGES::Order::API::Object::LineItemContainer::insertLineItem>.
#----------------------------------------------------------------------------------------
# input        @Parameter | list of parameters that define a line item | array
#========================================================================================
sub insertLineItem    {
    my $self = shift;
    my @Parameter = @_;
    $self->container->insertLineItem(@Parameter);
}

#========================================================================================
# function     updateTaxArea
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CustomerOrder->updateTaxArea;
#----------------------------------------------------------------------------------------
# example      $CustomerOrder->set({'ShippingAddress' => $NewShippingAddress});
#               if( $CustomerOrder->updateTaxArea ) {
#                   $CustomerOrder->recalculate;
#               }
#----------------------------------------------------------------------------------------
# description  Updates the tax area of the order based on the shipping address.
#               Sets the tax are to the customer tax area if a tax area is explicitely
#               for the customer.
#               Note: After $CustomerOrder->updateTaxArea you must call
#               $CustomerOrder->recalculate to update the price calculation.
#----------------------------------------------------------------------------------------
# return       $TaxChanged | true if tax area has changed, otherwise undef | boolean
#========================================================================================
sub updateTaxArea {
    my $self = shift;
    my $Shop = $self->getSite;
    my $Container = $self->container;

    # customer tax area takes precedence, but it's not set by default
    my $TaxArea = $Container->get('TaxArea') // $self->get('Customer')->get('TaxArea');

    my $ShippingAddress = $self->get('ShippingAddress') // $self->get('BillingAddress');
    my $ShippingAddressIsEU = defined $ShippingAddress ? IsEUAddress($ShippingAddress) : 0;
    my $ShopCountryID = GetTaxCountryIDForShop($Shop);
    my $ShippingCountryID = $Container->get('ShippingCountryID');
    $ShippingCountryID = $ShopCountryID  unless $ShippingCountryID;

    my $hShopInfo = $Shop->get([qw(DeliveryThresholdIsActive Address)]);
    my $ShopAddressIsEU = IsEUAddress($hShopInfo->{'Address'});

    # set TaxArea to undef because of DT and EU address. The correct Area will be found later by the address
    if( $hShopInfo->{'DeliveryThresholdIsActive'} and $ShopAddressIsEU and $ShippingAddressIsEU ) {
        $TaxArea = undef;
    }

    my $TaxMatrixForDeliveryThreshold = undef;
    unless (defined $TaxArea) {
        # use the correct TaxMatrix

        if((defined $ShippingAddress) and $hShopInfo->{'DeliveryThresholdIsActive'}) {
            $TaxMatrixForDeliveryThreshold = TaxMatrixByEUAddress($ShippingAddress);
        }
        # use shop tax matrix if we have no DT or no shipping address
        my $TaxMatrix = defined $TaxMatrixForDeliveryThreshold ? $TaxMatrixForDeliveryThreshold : $Shop->get('TaxMatrix');

        #zipcode
        my $Zipcode = defined $ShippingAddress ? $ShippingAddress->get('Zipcode') : undef;

        # get correct TaxArea by zip code
        $TaxArea = $ShippingCountryID
                 ? $TaxMatrix->taxAreaByCountry($ShippingCountryID, $Zipcode)
                 : $TaxMatrix->fallbackTaxArea;
    }

    # set TaxArea and correct also classes on container if DT is activated
    my $OldTaxArea = $Container->get('TaxArea');
    if (defined $TaxArea && 0 != Compare('TaxArea', $OldTaxArea, $TaxArea)) {
        $Container->set({'TaxArea' => $TaxArea});
        $self->_correctTaxClassesOnContainer($Container, $TaxMatrixForDeliveryThreshold) if defined $TaxMatrixForDeliveryThreshold;
        $self->recalculate;
        return 1;
    }
    return;

}
sub _correctTaxClassesOnContainer {
    my $self = shift;
    my ($Container, $NewTaxMatrix) = @_;

    my $LineItemClass = LoadClassByAlias('LineItem');
    my @LineItems = grep { $_->instanceOf($LineItemClass) } @{$self->container->children};

    foreach my $LineItem (@LineItems) {
        #delete Tax items. The lineitems will be added later by using recalculate
        if ($LineItem->instanceOf('LineItemTax')){
            $LineItem->delete();
            next;
        }

        my $OldTaxClass = $LineItem->get('TaxClass');
        next unless defined $OldTaxClass;

        my $NewTaxClass = $NewTaxMatrix->getTaxClassByAlias($OldTaxClass->alias) ;
        $LineItem->set({'TaxClass' => $NewTaxClass } );

    }
}
#========================================================================================
# function     updateTaxAreaDigital
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TaxDigitalChanged = $CustomerOrder->updateTaxAreaDigital();
#----------------------------------------------------------------------------------------
# description  Sets the order tax area digital based on the shipping address country
#               Default tax area digital is the tax area based on the shop address
#
#               Returns true, if calculation of tax has changed, because of changed
#               country in billing or shipping address
#----------------------------------------------------------------------------------------
# return       $TaxDigitalChanged | true if tax area or tax calculation has changed,
#                             otherwise undef | boolean
#========================================================================================
sub updateTaxAreaDigital {
    my $self = shift;
    my $Container = $self->container;
    return unless defined $Container;

    # try to find the correct taxareadigital for the user
    # order: basket/user/shopfallback
    my $TaxAreaDigital = $self->taxAreaDigitalByAddress();

    my $OldTaxAreaDigital = $Container->get('TaxAreaDigital');
    if (Compare('TaxAreaDigital', $OldTaxAreaDigital, $TaxAreaDigital)) {
        $Container->set({'TaxAreaDigital' => $TaxAreaDigital});
        $self->recalculate;
        return 1;
    }

    return;
}

#========================================================================================
# function     taxAreaDigitalByAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TaxAreaDigital = $Order->taxAreaDigitalByAddress();
#----------------------------------------------------------------------------------------
# description  Get the order tax area digital based on the shipping address country
#               Default tax area digital is the tax area based on the shop address
#
#               Returns the tax area digital for the order, or undef if the address
#               isn't in the EU
#----------------------------------------------------------------------------------------
# return       $TaxAreaDigital | tax area digital object | object
#========================================================================================
sub taxAreaDigitalByAddress {
    my $self = shift;
    my $Shop = $self->getSite();

    my $TaxAddress;

    # ORDER: do we have already a basket address?
    $TaxAddress = $self->get('ShippingAddress') // $self->get('BillingAddress');


    if (defined $TaxAddress) {
        return undef unless $TaxAddress->get('IsEU');
    }
    else {  # SHOP FALLBACK: we use the shop address to get the digital tax matrix
        return undef unless $Shop->get('Address')->get('IsEU');
        return $Shop->get('DefaultTaxAreaDigital');
    }

    my $AddressCountryID = $TaxAddress->get('CountryID');
    my $TaxAreaDigital = $Shop->get('TaxMatrixDigital')->taxAreaDigitalByCountry($AddressCountryID);

    return $TaxAreaDigital;
}

#========================================================================================
# function     getWebUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $WebUrl = $Object->getWebUrl($hParams);
#----------------------------------------------------------------------------------------
# description  Construct the web url of a given object dependend of the following parameters:
#----------------------------------------------------------------------------------------
# input        $hParams | attributes of new object
#                       <ul>
#                         <li>SSL - SSL web url - boolean</li>
#                         <li>Section - Storefront | Mobile | Admin - string</li>
#                       </ul> | hash ref
# return       $WebUrl | web url of the object | string
#========================================================================================
sub getWebUrl {
    my $self = shift;
    my ($hParams) = @_;
    my $WebUrl;

    if ($self->getSite()->get('IsUnityRegistered')) {
        $WebUrl = $self->getWebUrlUnity($hParams);
    } else {
        $WebUrl = $self->SUPER::getWebUrl($hParams);
    }

    return $WebUrl;
}

1;
