#========================================================================================
# package      DE_EPAGES::Product::API::ProductVariationService
# state        public
#----------------------------------------------------------------------------------------
# description  Service to extract information about variation attributes and
#               sub products from a master product.
#========================================================================================
package DE_EPAGES::Product::API::ProductVariationService;

use DE_EPAGES::Product::API::Product qw (GetVariationAttributes InsertMultipleSubProducts);

use strict;


#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $MasterProduct );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# input        $MasterProduct | A master product | DE_EPAGES::Product::API::Object::Product
# input        $LanguageID | language id for localized display values | integer
# return       $self | self | DE_EPAGES::Product::API::ProductVariationService
#========================================================================================
sub new {
    my $class = shift;
    my ($MasterProduct, $LanguageID) = @_;
    my $self  = bless {
        'masterProduct' => $MasterProduct,
        'languageID' => $LanguageID,
        'variationTypeValues' => $MasterProduct->getWithParent('VariationTypeValues', $LanguageID)
    }, $class;
    return $self;
}



#========================================================================================
# function     getSubProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       getSubProducts( $IncludeInvisible );
#----------------------------------------------------------------------------------------
# description  Returns all variation products from a master product
#----------------------------------------------------------------------------------------
# input        $IncludeInvisible | include invisible sub products in case of true |boolean 
# return       Array of variation products | array_ref
#========================================================================================
sub getSubProducts {
    my $self = shift;
    my ($IncludeInvisible) = @_;

    my $MasterProduct = $self->{'masterProduct'};
    my $aSubProducts = $MasterProduct->subProducts();
    if ($IncludeInvisible ne 'true') {
        $aSubProducts = [grep { $_->get('IsVisible') } @$aSubProducts];
    }

    return $aSubProducts;
}

#========================================================================================
# function     createSubProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       createSubProducts( $hObjectsForVariation );
#----------------------------------------------------------------------------------------
# description  Creates variation products for a master product
#----------------------------------------------------------------------------------------
# input        $hObjectsForVariation | product type objects for creating variation products |ref.hash
#========================================================================================
sub createSubProducts {
    my $self = shift;
    my ($hObjectsForVariation) = @_;

    my $hOptions = {};
    $hOptions->{'IsVisible'} = 1 if $hObjectsForVariation->{'VisibleAfterCreation'};
    InsertMultipleSubProducts($self->{'masterProduct'}, $hObjectsForVariation->{'Attributes'}, {}, $hOptions);

    return;

}

#========================================================================================
# function     getVariationsForSubproduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       getVariationsForSubproduct( $SubProduct, $APIBaseURL );
#----------------------------------------------------------------------------------------
# description  Returns details about the variation attributes for a variation product
#----------------------------------------------------------------------------------------
# input        $SubProduct | A variation product | DE_EPAGES::Product::API::Object::Product
# input        $APIBaseURL | The base url of the REST API | string
# return       Hash reference with variation product details | hash_ref
#========================================================================================
sub getVariationsForSubproduct {
    my $self = shift;
    my ($SubProduct, $APIBaseURL) = @_;

    my $IsStockOK = $SubProduct->get('StockLevel') > 0;
    my $IsAvailable = $SubProduct->get('IsAvailable');
    my $IsPurchasable = 0;
    $IsPurchasable = 1 if $IsStockOK &&  $IsAvailable;

    my $aAttributeSelection = GetVariationAttributes($SubProduct, $self->{'languageID'}, $self->{'variationTypeValues'});
    return {
        'attributeSelection' => $aAttributeSelection,
        'link' => {
            'rel' => 'variation',
            'href' => $APIBaseURL.'/api/products/'.$SubProduct->guid()
        },
        'additionalAttributes' => {
            'purchasable' => $IsPurchasable,
        }
   }
}



#========================================================================================
# function     getVariationAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       getVariationAttributes( );
#----------------------------------------------------------------------------------------
# description  Returns details about all variations attributes and their possible
#               values for the master product.
#----------------------------------------------------------------------------------------
# return       Array of all variations attributes of the master product | array_ref
#========================================================================================
sub getVariationAttributes {
    my $self = shift;

    my $aVariationTypeValues = $self->{'variationTypeValues'};
    my $aVariationAttributes = [
        map {
            $self->_getAttribute($_, $self->{'languageID'})
        }
        @$aVariationTypeValues
    ];

    return $aVariationAttributes;
}



sub _getAttribute {
    my $self = shift;
    my ($hAttribute, $LanguageID) = @_;

    my $Attribute = $hAttribute->{'Attribute'};
    my $aPossibleAttributeValues = $hAttribute->{'Values'};

    my $aPossibleAttributeValueDetails = [
        map {
                {
                    'value' => $_->alias(),
                    'displayValue' => $_->get('ValueOrAlias', $LanguageID),
                }
        } @$aPossibleAttributeValues
    ];

    return {
        'name' => $Attribute->alias(),
        'displayName' => $Attribute->get('NameOrAlias', $LanguageID),
        'values' => $aPossibleAttributeValueDetails,
    };
}

1;
