#========================================================================================
# package      DE_EPAGES::Product::t::unit::ProductCollectionResource
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Product::API::ProductVariationService
#========================================================================================
package DE_EPAGES::Product::t::unit::ProductVariationService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Product::API::ProductVariationService;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;

use strict;


sub test {
    my $self = shift;

    $self->test_ProductVariationService_getSubProducts();
    $self->test_ProductVariationService_getSubProducts_includeInvisible();
    $self->test_ProductVariationService_getVariationsForSubproduct();
    $self->test_ProductVariationService_getVariationAttributes();

    done_testing();
}


sub test_ProductVariationService_getSubProducts {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $MasterProduct = $Mockery->get('MasterProduct');

    my $ProductVariationService = DE_EPAGES::Product::API::ProductVariationService->new($MasterProduct);

    my $aVisibleSubProducts = $ProductVariationService->getSubProducts();
    is(scalar @$aVisibleSubProducts, 2, "$SubTestName - test if 2 visible subproduct were returned.");
    is($aVisibleSubProducts->[0]->alias(), 'TestProductA', "$SubTestName - test if the right product was returned.");
    is($aVisibleSubProducts->[1]->alias(), 'TestProductC', "$SubTestName - test if the right product was returned.");
}

sub test_ProductVariationService_getSubProducts_includeInvisible {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $MasterProduct = $Mockery->get('MasterProduct');

    my $ProductVariationService = DE_EPAGES::Product::API::ProductVariationService->new($MasterProduct);

    my $IncludeInvisible = 'true';
    my $aSubProducts = $ProductVariationService->getSubProducts($IncludeInvisible);
    is(scalar @$aSubProducts, 3, "$SubTestName - test if 3 subproduct were returned.");
    is($aSubProducts->[0]->alias(), 'TestProductA', "$SubTestName - test if the right product was returned.");
    is($aSubProducts->[1]->alias(), 'TestProductC', "$SubTestName - test if the right product was returned.");
    is($aSubProducts->[2]->alias(), 'TestProductB', "$SubTestName - test if the right product was returned.");
}

sub test_ProductVariationService_getVariationsForSubproduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $MasterProduct = $Mockery->get('MasterProduct');

    my $ProductVariationService = DE_EPAGES::Product::API::ProductVariationService->new($MasterProduct);

    my $SubProduct = $Mockery->get('SubProduct');
    my $aSubProductVariations = $ProductVariationService->getVariationsForSubproduct($SubProduct, 'http://www.api-base.url');
    is_deeply($aSubProductVariations, $self->_createExpectedSubProduct1Variations(), "$SubTestName - test right variations were returned");

    my $SubProduct2 = $Mockery->get('SubProduct2');
    my $aSubProduct2Variations = $ProductVariationService->getVariationsForSubproduct($SubProduct2, 'http://www.api-base.url');
    is_deeply($aSubProduct2Variations, $self->_createExpectedSubProduct2Variations(), "$SubTestName - test right variations were returned");
}


sub test_ProductVariationService_getVariationAttributes {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $MasterProduct = $Mockery->get('MasterProduct');

    my $ProductVariationService = DE_EPAGES::Product::API::ProductVariationService->new($MasterProduct);

    my $aVariationAttributes = $ProductVariationService->getVariationAttributes();
    is_deeply($aVariationAttributes, $self->_createExpectedVariationAttributes(), "$SubTestName - test right variation attributes were returned");
}


sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->extend($self->_createProductMocks());
    $Mockery->extend($self->_createAttributeMocks());

    $Mockery->register('MasterProduct', [
        'AttributeSize', 'AttributeSizeM', 'AttributeSizeL',
        'AttributeColor', 'AttributeColorBlack', 'AttributeColorWhite',
        'SubProduct', 'SubProduct2', 'SubProduct3'
        ], sub {
        my (
            $AttributeSize, $AttributeSizeM, $AttributeSizeL,
            $AttributeColor, $AttributeColorBlack, $AttributeColorWhite,
            $SubProduct, $SubProduct2, $SubProduct3
        ) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'VariationTypeValues' => [
                {
                    Attribute => $AttributeSize,
                    Values => [$AttributeSizeM, $AttributeSizeL]
                },
                {
                    Attribute => $AttributeColor,
                    Values => [$AttributeColorBlack, $AttributeColorWhite]
                },
            ],
        });
        $Mockify->addMockWithReturnValue('subProducts', [$SubProduct, $SubProduct2, $SubProduct3]);
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}


sub _createProductMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SubProduct', ['AttributeSizeM', 'AttributeColorBlack'], sub {
        my ($AttributeSizeM, $AttributeColorBlack) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'Alias'               => 'TestProductA',
            'GUID'                => '314-159',
            'IsVisible'           => 1,
            'Size'                => $AttributeSizeM,
            'Color'               => $AttributeColorBlack,
            'VariationTypeValues' => [],
            'ImageInformations'   => {},
            'StockLevel'          => 1,
            'IsAvailable'         => 1,
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('SubProduct2', ['AttributeSizeL', 'AttributeColorWhite'], sub {
        my ($AttributeSizeL, $AttributeColorWhite) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'Alias' => 'TestProductC',
            'GUID' => '123-456',
            'IsVisible' => 1,
            'Size' => $AttributeSizeL,
            'Color' => $AttributeColorWhite,
            'VariationTypeValues'=> [],
            'ImageInformations' => {},
            'StockLevel'          => 1,
            'IsAvailable'         => 1,
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('SubProduct3', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'Alias' => 'TestProductB',
            'IsVisible' => 0,
            'ImageInformations' => {},
            'StockLevel'          => 1,
            'IsAvailable'         => 1,
        });
        return $Mockify->getMockObject();
    });

    return $Mockery;
}


sub _createAttributeMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('AttributeSize', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::ProductAttribute', {
            'Alias' => 'Size',
            'NameOrAlias' => 'Groesse',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('AttributeSizeM', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::PreDefAttribute', {
            'Alias' => 'M',
            'ValueOrAlias' => 'normal',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('AttributeSizeL', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::PreDefAttribute', {
            'Alias' => 'L',
            'ValueOrAlias' => 'gross',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('AttributeColor', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::ProductAttribute', {
            'Alias' => 'Color',
            'NameOrAlias' => 'Farbe',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('AttributeColorBlack', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::PreDefAttribute', {
            'Alias' => 'black',
            'ValueOrAlias' => 'schwarz',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('AttributeColorWhite', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::PreDefAttribute', {
            'Alias' => 'white',
            'ValueOrAlias' => 'weiss',
        });
        return $Mockify->getMockObject();
    });

    return $Mockery;
}


sub _createExpectedSubProduct1Variations {
    return {
        'link' => {
            'rel' => 'variation',
            'href' => 'http://www.api-base.url/api/products/314-159',
        },
        'additionalAttributes' => {
            'purchasable' => 1
        },
        'attributeSelection' => [
            {
                'displayValue' => 'normal',
                'value' => 'M',
                'name' => 'Size',
                'displayName' => 'Groesse'
            },
            {
                'displayValue' => 'schwarz',
                'value' => 'black',
                'name' => 'Color',
                'displayName' => 'Farbe'
            }
        ]
    };
}


sub _createExpectedSubProduct2Variations {
    return {
        'link' => {
            'rel' => 'variation',
            'href' => 'http://www.api-base.url/api/products/123-456',
        },
        'additionalAttributes' => {
            'purchasable' => 1
        },
        'attributeSelection' => [
            {
                'displayValue' => 'gross',
                'value' => 'L',
                'name' => 'Size',
                'displayName' => 'Groesse'
            },
            {
                'displayValue' => 'weiss',
                'value' => 'white',
                'name' => 'Color',
                'displayName' => 'Farbe'
            }
        ]
    };
}


sub _createExpectedVariationAttributes {
    return [
        {
            'name' => 'Size',
            'values' => [
                {
                    'displayValue' => 'normal',
                    'value' => 'M'
                },
                {
                    'displayValue' => 'gross',
                    'value' => 'L'
                }
            ],
            'displayName' => 'Groesse'
        },
        {
            'name' => 'Color',
            'values' => [
                {
                    'displayValue' => 'schwarz',
                    'value' => 'black'
                },
                {
                    'displayValue' => 'weiss',
                    'value' => 'white'
                }
            ],
            'displayName' => 'Farbe'
        }
    ]
}

__PACKAGE__->runScript();